unit VitalsData;

{===============================================================================
 VitalsData
    This unit was created in response to an issue with the vital statistics
      graphing in graphVS.pas.  The issue occurred when data added to the graph
      was then sorted internally in the TChart component, it did not agree
      with the order stored in stringlists used to store the hint value of
      various points.  Thus the displayed hint values were not in agreement with
      the plotted points

  Classes
    TVitalsDataPoint
      A data storage class used to store the plot points.  These plot point are
        used to populate the TChart
    TVitalsData:
      A class used to collect and sort the plot points.  This sorted data is
        used to populate the TChart in graphVS

  Modification History:
    CodeCR128 - Unit created 3/2011 -MER
 ==============================================================================}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Contnrs, Chart;

type
  TVitalsDataPoint = class(TObject)
  public
    ySeries1: integer;
    ySeries2: integer;
    DateTime: TDateTime;
    constructor Create(ADateTime: TDateTime; AySeries1: integer; AySeries2: integer);
  end;

  TVitalsData = class(TObject)
  private
    FIsDualSeries: Boolean;
    FPoints: TObjectList;
    FSorted: Boolean;
    procedure SetSorted(Value: Boolean);
  public
    property IsDualSeries: Boolean read FIsDualSeries write FIsDualSeries;
    property Points: TObjectList read FPoints;
    property Sorted: Boolean read FSorted write SetSorted;
    constructor Create();
    destructor Destroy(); override;
    function AddPoint(ADate: string; ATime: string; AySeries1: string; AySeries2: string): Boolean;
    procedure Clear();
    function GetMarkText(ValueIndex: Longint; Series: integer): String;
    procedure LoadPointsToChart(VitalsChart: TChart);
  end;

implementation

{===============================================================================
 CompareVDPDate
    CompareVDPDate is a compare function that is used for the Points objectList
      sort.  It compares the dateTimes of the 2 passed items and returns an
      integer representing less than, greater than, or equal

  Parameters
    Item1:
      A pointer to the first TVitalsDataPoint object for comparison
    Item2:
      A pointer to the second TVitalsDataPoint object for comparison

  Return values
    CompareVDPDate returns an integer representing if a swap is needed.
      If the first item is lesser, it returns -1.  If the first item is greater,
      it returns 1.  If they are equal, it returns 0

  Modification History:
    CodeCR128 - Method created 3/2011 -MER
 ==============================================================================}
function CompareVDPDate(Item1, Item2: Pointer): Integer;
var
  date1: TDateTime;
  date2: TDateTime;
begin
  date1 := TVitalsDataPoint(Item1).DateTime;
  date2 := TVitalsDataPoint(Item2).DateTime;
  Result := 0;
  if date1 < date2 then
    Result := -1
  else if date1 > date2 then
    Result := 1;
end;

////////////////////////////////////////////////////////////////////////////////
// Supporting class TVitalsDataPoint definition
////////////////////////////////////////////////////////////////////////////////

{===============================================================================
  TVitalsDataPoint constructor

  Parameters
    ADateTime:
      TDateTime of the vitals data point
    AySeries1:
      Integer representing the series 1 data
    AySeries2:
      Integer representing the series 2 data

  Modification History:
    CodeCR128 - Method created 3/2011 -MER
 ==============================================================================}
constructor TVitalsDataPoint.Create(ADateTime: TDateTime; AySeries1: integer; AySeries2: integer);
begin
  inherited Create;
  DateTime := ADateTime;
  ySeries1 := AySeries1;
  ySeries2 := AySeries2;
end;

////////////////////////////////////////////////////////////////////////////////
// TVitalsData definition
////////////////////////////////////////////////////////////////////////////////

{===============================================================================
  SetSorted
    SetSorted is setter method for property Sorted.  If Sorted is true, a call
      is made to sort FPoints

  Modification History:
    CodeCR128 - Method created 3/2011 -MER
 ==============================================================================}
procedure TVitalsData.SetSorted(Value: Boolean);
begin
  FSorted := Value;
  if FSorted then
    FPoints.Sort(CompareVDPDate);
end;

{===============================================================================
  TVitalsData constructor

  Modification History:
    CodeCR128 - Method created 3/2011 -MER
 ==============================================================================}
constructor TVitalsData.Create();
begin
  inherited Create;
  FIsDualSeries := FALSE;
  FSorted := TRUE;
  FPoints := TObjectList.Create;
end;

{===============================================================================
  TVitalsData destructor
    This method clears and then frees TObjectList FPoints.  The memory of the
      objects owned by FPoints are released by the TObjectList code

  Modification History:
    CodeCR128 - Method created 3/2011 -MER
 ==============================================================================}
destructor TVitalsData.Destroy();
begin
  if FPoints <> nil then begin
    FPoints.Clear; // note: the TObjectList frees the associated items
    FPoints.Free;
  end;
  inherited Destroy;
end;

{===============================================================================
 AddPoint
    AddPoint is function that adds points of vital statistics data to the Points
      property of a VitalsData object.  It parses the passed string information
      and generates TDateTime and integer values.  A VitalsDataPoint object is
      created and then stored in Points.  If sorting is TRUE, the Points are
      sorted after insertion

  Parameters
    ADate:
      A string representing the date.  This is used in conjunction with ATime to
        get the date and time
    ATime:
      A string representing the time
    AySeries1:
      A string representing the series 1 vital statistic value
    AySeries2:
      A string representing the series 2 vital statistic value

  Return values
    AddPoint returns a boolean representing success.  If any of the parameter
      parsing generates an exception, result is FALSE, otherwise it returns
      TRUE.  At the time this class was created, in keeping with the existing
      business logic, there is no branching based on return value.  The return
      value was provided in anticipation of possible future need

  Modification History:
    CodeCR128 - Method created 3/2011 -MER
 ==============================================================================}
function TVitalsData.AddPoint(ADate: string; ATime: string; AySeries1: string; AySeries2: string): Boolean;
var
  y1: integer;
  y2: integer;
  lDate: TDateTime;
  lTime: TDateTime;
  lDateTime: TDateTime;
begin
  Result := TRUE;
  try
    lDate := VarToDateTime(ADate);
  except
    on E:Exception do begin
      lDate := Trunc(Now);
      Result := FALSE;
    end;
  end;
  try
    lTime := StrToTime(ATime);
  except
    on E:Exception do begin
      lTime := 0;
      Result := FALSE;
    end;
  end;
  // the lTime day should be 0, but just in case, we'll strip the day
  lDateTime := Trunc(lDate) + (lTime - Trunc(lTime));
  try
    y1 := StrToInt(AySeries1);
  except
    on E:Exception do begin
      y1 := 0;
      Result := FALSE;
    end;
  end;
  try
    y2 := StrToInt(AySeries2);
  except
    on E:Exception do begin
      y2 := 0;
      Result := FALSE;
    end;
  end;
  FPoints.Add(TVitalsDataPoint.Create(lDateTime, y1, y2));
  if FSorted then
    FPoints.Sort(CompareVDPDate);
end;

{===============================================================================
  Clear
    Clear is a method that clears FPoints. The memory of the objects owned by
      FPoints is released by the TObjectList code

  Modification History:
    CodeCR128 - Method created 3/2011 -MER
 ==============================================================================}
procedure TVitalsData.Clear();
begin
  FPoints.Clear; // note: the TObjectList frees the associated items
end;

{===============================================================================
 GetMarkText
    GetMarkText is a function that returns a string representing the value of
      the requested vitals data point

  Parameters
    ValueIndex:
      A LongInt representing the index of the requested vitals data point
    Series:
      An integer representing the series of the point requested.  At the
        present, the valid series values are either 1 or 2

  Return values
    GetMarkText returns a string representing the value of the requested vitals
      data point.  If a request cannot be satisfied, it returns an empty string

  Modification History:
    CodeCR128 - Method created 3/2011 -MER
 ==============================================================================}
function TVitalsData.GetMarkText(ValueIndex: Longint; Series: integer): String;
begin
  if ValueIndex > (FPoints.Count - 1) then begin
    Result := '';
  end else begin
    if Series = 1 then
      Result := IntToStr(TVitalsDataPoint(FPoints[ValueIndex]).ySeries1)
    else if Series = 2 then
      Result := IntToStr(TVitalsDataPoint(FPoints[ValueIndex]).ySeries2)
    else
      Result := '';
  end;
end;

{===============================================================================
 LoadPointsToChart
    LoadPointsToChart is a procedure that loads the vitals data points into a
      TChart component.  If the data has 2 series (such as blood pressure), both
      series of data are loaded.

  Parameters
    VitalsChart:
      A TChart to which current data points will be loaded

  Modification History:
    CodeCR128 - Method created 3/2011 -MER
 ==============================================================================}
procedure TVitalsData.LoadPointsToChart(VitalsChart: TChart);
var
  ndx: integer;
  lVDP: TVitalsDataPoint;
begin
  for ndx := 0 to (FPoints.Count - 1) do begin
    lVDP := TVitalsDataPoint(FPoints[ndx]);
    if FIsDualSeries then begin
      VitalsChart.Series[0].AddXY(lVDP.DateTime, lVDP.ySeries1);
      VitalsChart.Series[1].AddXY(lVDP.DateTime, lVDP.ySeries2);
    end else begin
      VitalsChart.Series[0].AddXY(lVDP.DateTime, lVDP.ySeries1);
    end;
  end;
end;

end.
