unit DateRangeSelect;

{==============================================================================
  TFormDateRange
    TFormDateRange is a form that will set filtering criteria for use on the
      main form of CAPRI.  The original form was going to be modified for 508
      compliance.  However, it was learned that Ron and Jesse had already
      protyped a replacement form that would be improved for both traditional
      and screen reader use.  This prototype replaced the existing form and was
      finished up for implemetation in patch 149.

  Modification History:
    CodeCR122 - 9/2010 Form replaced with one created by RPM -MER
 ==============================================================================}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ComCtrls, VA508AccessibilityManager, AccessibilityUtils,
  VAUtils, DialogsCapri;

type
  EBadDates = Class(Exception);
  TFormDateRange = class(TForm)
    pnlSortChoice: TPanel;
    gbxSelectSortType: TGroupBox;
    rbtnByDate: TRadioButton;
    rbtnNumber: TRadioButton;
    rbtnAll: TRadioButton;
    grpStartDate: TGroupBox;
    grpEndDate: TGroupBox;
    btnOK: TButton;
    btnCancel: TButton;
    dtpStartDate: TDateTimePicker;
    dtpEndDate: TDateTimePicker;
    grbMaxNumber: TGroupBox;
    edbNumber: TEdit;
    UpDown1: TUpDown;
    VA508AccessibilityManager1: TVA508AccessibilityManager;
    st508StartDate: TVA508StaticText;
    st508EndDate: TVA508StaticText;
    st508MaxRecords: TVA508StaticText;
    procedure btnOKClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure rbtnAllClick(Sender: TObject);
    procedure rbtnNumberClick(Sender: TObject);
    procedure rbtnByDateClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure dtpStartDateExit(Sender: TObject);
    procedure edbNumberKeyPress(Sender: TObject; var Key: Char);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    ScreenReaderItemsActive: Boolean;
    procedure Update508Components;
  public
    { Public declarations }
  end;

var
  FormDateRange: TFormDateRange;

implementation

{$R *.dfm}

{==============================================================================
  btnCancelClick
    btnCancelClick is an event handler that closes the form with a modal result
      of mrCancel;

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
end;

{==============================================================================
  btnOKClick
    btnOKClick is an event handler that validates the user input based on the
      filter type.  If the validation passes, the form is closed with a modal
      result of mrOK;

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.btnOKClick(Sender: TObject);
begin
  try
    // validate the input based on the filter type
    if rbtnByDate.Checked then begin
      if (dtpStartDate.Date > dtpEndDate.Date) then
        raise EBadDates.Create('The start date must be less than or equal to the end date');
    end else if rbtnNumber.Checked then begin
      StrToInt(edbNumber.Text);
    end;
    ModalResult := mrOK;
  except
    on E:Exception do
      ShowMessage(E.Message);
  end;
end;

{==============================================================================
  dtpStartDateExit
    dtpStartDateExit is an event handler that helps ensure that a valid date
      pair is selected

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.dtpStartDateExit(Sender: TObject);
begin
  if (dtpStartDate.Date > dtpEndDate.Date) then
    ShowMessageCAPRI('The start date must be less than or equal to the end date');
end;

{==============================================================================
  edbNumberKeyPress
    edbNumberKeyPress is an event handler that limits keystrokes to valid
      numeric and control input.  Note that keypress does not get called for
      virtual keys, so the only control key handled is backspace.
    If an invalid key is processed, it is set to 0 (killing the keystroke) and
      a messageBeep is called

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.edbNumberKeyPress(Sender: TObject; var Key: Char);
begin
  if not(Key in [#8, '0'..'9'])
  then begin
    Key := #0;
    MessageBeep(0);
  end;
end;

{==============================================================================
  FormCreate
    FormCreate is an event handler that initializes the form.  The dates are
      initialized to today, the filter criteria is by number, the default
      number is 100.  We check to see if a screen reader is active and set
      ScreenReaderItemsActive as well as the visibility of the 508 items

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.FormCreate(Sender: TObject);
begin
  dtpStartDate.Date := NOW;
  dtpEndDate.Date := NOW;
  rbtnNumber.Checked := True;
  UpDown1.Position := 100;
  if VAUtils.ScreenReaderActive then begin
    ScreenReaderItemsActive := TRUE;
    Update508Components;
  end else begin
    ScreenReaderItemsActive := FALSE;
    st508StartDate.Visible := FALSE;
    st508EndDate.Visible := FALSE;
    st508MaxRecords.Visible := FALSE;
  end;
end;

{==============================================================================
  FormShow
    FormShow is an event handler that will ensure that the item with focus is
    the radio button currently checked

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.FormShow(Sender: TObject);
begin
  if rbtnByDate.Checked then
    rbtnByDate.SetFocus
  else if rbtnNumber.Checked then
    rbtnNumber.SetFocus
  else
    rbtnAll.SetFocus;
end;

{==============================================================================
  rbtnAllClick
    rbtnAllClick is an event handler that will update the state of UI
    components based on the filter criteria being set to show all

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.rbtnAllClick(Sender: TObject);
begin
  grpStartDate.Enabled := False;
  grpEndDate.Enabled := False;
  dtpStartDate.Enabled := False;
  dtpEndDate.Enabled := False;
  edbNumber.Enabled := False;
  UpDown1.Enabled := False;
  Update508Components;
end;

{==============================================================================
  rbtnByDateClick
    rbtnByDateClick is an event handler that will update the state of UI
    components based on the filter criteria being set to filter by date

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.rbtnByDateClick(Sender: TObject);
begin
  grpStartDate.Enabled := True;
  grpEndDate.Enabled := True;
  dtpStartDate.Enabled := True;
  dtpEndDate.Enabled := True;
  edbNumber.Enabled := False;
  UpDown1.Enabled := False;
  Update508Components;
end;

{==============================================================================
  rbtnNumberClick
    rbtnNumberClick is an event handler that will update the state of UI
    components based on the filter criteria being set to filter by number of docs

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.rbtnNumberClick(Sender: TObject);
begin
  grpStartDate.Enabled := False;
  grpEndDate.Enabled := False;
  dtpStartDate.Enabled := False;
  dtpEndDate.Enabled := False;
  edbNumber.Enabled := Enabled;
  UpDown1.Enabled := Enabled;
  Update508Components;
end;

{==============================================================================
  Update508Components
    Update508Components is a method that will update the visibility of the 508
      compliance components on this form.  This visibility is based on the
      state of the companion UI components

  Modification History:
    CodeCR122 - 9/2010 Method created -MER
 ==============================================================================}
procedure TFormDateRange.Update508Components;
begin
  if ScreenReaderItemsActive then begin
    if (grpStartDate.Enabled) then begin
      st508StartDate.Visible := FALSE;
    end else begin
      st508StartDate.Caption := 'Start Date.  ' + DateToStr(dtpStartDate.Date) + ST_DISABLED;
      st508StartDate.Visible := TRUE;
    end;
    if (grpEndDate.Enabled) then begin
      st508EndDate.Visible := FALSE;
    end else begin
      st508EndDate.Caption := 'End Date.  ' + DateToStr(dtpEndDate.Date) + ST_DISABLED;
      st508EndDate.Visible := TRUE;
    end;
    if (edbNumber.Enabled) then begin
      st508MaxRecords.Visible := FALSE;
    end else begin
      st508MaxRecords.Caption := 'Max records Included.  ' + edbNumber.Text + ST_DISABLED;
      st508MaxRecords.Visible := TRUE;
    end;
  end;
end;

end.
